/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;


/**
 * This class represents a role or, more exactly, the role projection
 * over some individual, as a fuzzy set
 * @author marco furlan
 * 
 */
public class Role implements FuzzySet {
	
	/**
	 * The map between Individuals and membership values
	 */
	private  Map<Individual, Double> r;
	/**
	 * The role name
	 */
	private String name;
	/**
	 * The KnowledgeBase
	 */
	private KnowledgeBase kb; 

	
	/**
	 * Constructs this Role by name and KnowledgeBase
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public Role(String name, KnowledgeBase kb) {
		this.setKb(kb);	
		this.setR(new TreeMap<Individual, Double>());
		this.setName(name);
		this.getKb().addRole(this);
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	
	/**
	 * Constructs this Role by map and KnowledgeBase
	 * @param r a map containing Individuals and membership values
	 * @param kb the KnowledgeBase
	 */
	public Role(TreeMap<Individual, Double> r, KnowledgeBase kb) {
		this.setKb(kb);
		this.setR(r);
		this.setName("role");
		this.getKb().addRole(this);
	}

	/**
	 * Constructs an empty Role
	 * @param kb the KnowledgeBase
	 */
	public Role(KnowledgeBase kb) {			
		this.setKb(kb);	
		this.setR(new TreeMap<Individual, Double>());		
		this.setName("role");
		this.getKb().addRole(this);
	}
	
	/**
	 * Adds a [Individual, value] couple to this Role
	 * @param s the Individual
	 * @param d the membership value
	 * @return this role after adding point
	 */
	public Role add(Individual s, Double d){
		this.r.put(s, d);
		return this;
	}
	
	/**	 
	 * The cardinality of this Role at an alpha level
	 * @param a the alpha value
	 * @return the integer number cardinality of the alpha-set
	 */
	public Integer card(Double a) {
		Set<Individual> keys =this.r.keySet();
		Iterator<Individual> iter= keys.iterator();
		int card=0;
		while(iter.hasNext()){
			Individual key =(Individual) iter.next();
			Double value=this.r.get(key);
			if (value>=a){
				card++;
			}		
		}	
		return card;
	}
	
	/**
	 * Gets the union set of alpha-values belonging to this Role and to a Concept
	 * @param c the Concept
	 * @return the union set of alpha-values
	 */
	public TreeSet<Double> totalLambda(Concept c) {
		TreeSet<Double> l = getLambda();
		if (c!=null){
			l.addAll(c.getLambda());
		}		
		return l;
	}
	/**
	 * Gets the union set of alpha-values belonging to this Role and to 
	 * a second Role
	 * @param c the second Role
	 * @return the union set of alpha-values
	 */
	public TreeSet<Double> totalLambda(Role c) {
		TreeSet<Double> l = getLambda();
		if (c!=null){
			l.addAll(c.getLambda());
		}		
		return l;
	}
	/**
	 * Gets the set lambda of alpha-values belonging to this Role 
	 * @return the set of alpha-values
	 */
	public TreeSet<Double> getLambda() {
		TreeSet<Double> l = new TreeSet<Double>( this.getR().values());		
		l.add(1.0);
		return l;
	}
	
//	/**
//	 * @param f
//	 * @return
//	 */
//	public Role sum(FuzzySet f){
//		// da fare
//		return null;
//	}
//	/**
//	 * @param f
//	 * @return the difference with a FuzzySet
//	 */
//	public Role diff(FuzzySet f){
//		//da fare
//		return null;
//	}
	/**
	 * @param f
	 * @return the product with a FuzzySet
	 */
//	public Role prod(FuzzySet f){
//	// da fare
//	return null;
//	}
//	/**
//	 * @param f
//	 * @return the ratio with a FuzzySet
//	 */
//	public Role ratio(FuzzySet f){
//		//da fare
//		return null;
//	}
//	/**
//	 * @param f
//	 * @return the intersection with a FuzzySet
//	 */
//	public Role intersection(FuzzySet f){
//		//da fare
//		return null;
//	}

	/**
	 * Gets the set of Individuals contained in this Role
	 * @return the set of Individuals
	 */
	public Set<Individual> getInds(){
		Set<Individual> t = (Set<Individual>) (getR().keySet());
		return t;
	}
	/**
	 * Tests the given Individual existence in this Role
	 * @param key the Individual
	 * @return <code>true</code> if this Role contains key, <code>false</code> otherwise
	 */
	public boolean containsInd(Individual key) {		
		return this.r.containsKey(key);
	}

	/**
	 * Intersection between this Role and a Concept
	 * @param c the Concept
	 * @return a new Role intersection 
	 */
	public Role intersection(Concept c) {
		Role inter = new Role(getKb());
		TreeSet<Individual> rKeys= new TreeSet<Individual>(getInds());
		TreeSet<Individual> keys=rKeys;
		Individual key;
		Double rVal, cVal, minVal;
		Iterator<Individual> it=keys.iterator();
		while (it.hasNext()){
			key=(Individual)it.next();			
			if ((c.containsKey(key))){
				rVal=(Double)this.get(key);			
				cVal=(Double)c.get(key);			
				minVal=new Double(Math.min(rVal,cVal));	
				if (minVal!=0){
					inter.add((Individual) key,minVal);	
				}
			}			
		}
		return inter;
	}

	/**
	 * Concatenation between this Role and an other one Role
	 * @param r1 the second Role
	 * @return a new Role concatenation 
	 */
	public Role concat(Role r1) {
		Role concat = new Role(this.getName()+"."+r1.getName(),getKb());
		TreeSet<Individual> rKeys= new TreeSet<Individual>(getInds());
		TreeSet<Individual> keys=rKeys;
		Double rVal, r1Val, minVal; 
		for (Individual ind : keys){
			if ((r1.containsInd(ind))){
				rVal=(Double)this.get(ind);			
				r1Val=(Double)r1.get(ind);			
				minVal=new Double(Math.min(rVal,r1Val));	
				if (minVal!=0){
					concat.add((Individual)ind, minVal);	
				}
			}			
		}
		return concat;
	}
	/**
	 * Gets membership value for a given Individual
	 * @param ind the Individual
	 * @return the membership value
	 */
	public double get(Individual ind) {
		return this.r.get(ind);
	}


	/**
	 * Prints a string representation for this Role
	 */
	public  void print() {
		Set<Individual> keys =getInds();
		Iterator<Individual> iter= keys.iterator();
		while(iter.hasNext()){
			Individual key =(DynamicInd)iter.next();
			Double value= (Double)get(key);
			if(key instanceof DynamicInd )
				System.out.printf("%s --> %.2f;  ",key.getName(), value);
			else 
				System.out.printf("%.2f --> %.2f;  ",key, value);
		}
		System.out.println("");
	}

	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}

	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}

	/**
	 * @return the r
	 */
	public Map<Individual, Double> getR() {
		return r;
	}

	/**
	 * @param r the r to set
	 */
	public void setR(Map<Individual, Double> r) {
		this.r = r;
	}
	/** 
	 * A String representation for this Role
	 * @return the String
	 */
	public String toString() {
		return "Role: " + 
		getName()+ "\n";
	}

	/* (non-Javadoc)
	 * @see reasoner.FuzzySet#card(java.lang.Double, java.lang.String)
	 */
	@Override
	public Integer card(Double alpha, String mode) {
		if (mode.equals("any"))
			return this.card(alpha);
		else{
			//to be implemented
			return 0;
		}
	}

}
