/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import util.Point;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import util.LinearInterpolation;


/**
 * This class represents a logic Concept as a FuzzySet of Individual
 * @author marco furlan
 */
public class Concept implements FuzzySet {
	/**
	 * the map of EventInd and values
	 */	
	protected Map<Individual, Double> events;
	/**
	 * the map of ULineInd and values
	 */
	protected Map<Individual, Double> uLineMap;	
	/**
	 * the map of StaticInd and values
	 */
	protected Map<Individual, Double> staticMap;	
	/**
	 * the sometime map
	 */	
	private Map<Individual, Double> sometimeMap;
	/**
	 * the always map
	 */	
	private Map<Individual, Double> alwaysMap;
//	/**
//	 * utility concept
//	 */	
//	private Concept r;
//	/**
//	 * utility concept
//	 */	
//	private Concept a1; 
	/**
	 * the concept name
	 */
	private String name;
	/**
	 * the Knowledge Base
	 */
	protected static KnowledgeBase kb;
	/**
	 * the constant _now_
	 */
	protected static double NOW;
	/**
	 * the constant _self_
	 */
	protected static double SELF;
	/**
	 * utility formatter
	 */
	protected NumberFormat formatter; 


	/**
	 * Constructs an empty Concept, with a given name and in a given KnowledgeBase
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public Concept(String name, KnowledgeBase kb) {
		setKb(kb);
		setName(name);
		setEvents(new TreeMap<Individual, Double>());			
		setULineMap(new TreeMap<Individual, Double>());
		setStaticMap(new TreeMap<Individual, Double>());
		setSometimeMap(new TreeMap<Individual, Double>());
		setAlwaysMap(new TreeMap<Individual, Double>());
		getKb().addConcept(this);
		setFormatter(3);
	}

	/**
	 * Constructs a Concept from the given map of EventInds and values and with
	 * a given name and in a given knowledge base
	 * @param ev the given map of EventInds and values
	 * @param name the name
	 * @param kb the knowledge base
	 */
	public Concept(Map<Individual, Double> ev, String name, KnowledgeBase kb) {
		setKb(kb);
		setEvents(ev);
		setULineMap(new TreeMap<Individual, Double>());
		setStaticMap(new TreeMap<Individual, Double>());
		setName(name);
		getKb().addConcept(this);
		setFormatter(3);		
	}
	private void setFormatter(int number){
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(number);
	}
	
	/**
	 * Gives this Concept name
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets this Concept name
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the membership value of the given Individual on the
	 * logical expression [sometimeA(ind)], where A is this Concept, using
	 * the 'sup' semantics.
	 * @param ind the given Individual
	 * @return the membership value
	 */
	public double sometime (Individual ind) {
		ULineEvent ul;
		Individual old, ind2;
		double val, t1, t2;
		ul = ((DynamicInd)ind).getULineEvent();
		t1 = ind.getTime();
		old = this.getLastSometime(ul);
//		if (t1 == getSELF())
		if (ind instanceof ULineInd)
			t1=ul.getLast().getTime();		
		if(old == null){
			ind2 = ul.getFirst();								
			val = ind2.degree(this);
		}else{
			ind2 = old;
			val=this.getSometimeMap().get(ind2);
		}	
		t2 = ind2.getTime();
		if (t1 < t2){			//from the beginning
			ind2 = ul.getFirst();						
			val = ind2.degree(this);
			t2 = ind2.getTime();
		}		
		while (t2<t1){
			ind2 = ul.getNext(ind2);
			if (ind2!=null){
				val = Math.max(val, ind2.degree(this));
				t2 = ind2.getTime();
			}
		}
		if (t2==t1){
			if (old != null && !old.equals(ind2)){
				this.getSometimeMap().remove(old);
				this.getSometimeMap().put(ind2, val);
			}else if (old == null)
				this.getSometimeMap().put(ind2, val);
			return val;
		}		
		return 0.0;
	}

	/**
	 * Returns the membership value of the given Individual on the
	 * logical expression [alwaysA(ind)], where A is this Concept, using
	 * the 'inf' semantics.
	 * @param ind the given Individual
	 * @return the membership value
	 */
	public double always (Individual ind) {
		ULineEvent ul;
		Individual old, ind2;
		double val, t1, t2;
		ul = ((DynamicInd)ind).getULineEvent();
		t1 = ind.getTime();
		old = this.getLastAlways(ul);
//		if (t1 == getSELF())
		if (ind instanceof ULineInd)
			t1=ul.getLast().getTime();	
		if(old == null){
			ind2 = ul.getFirst();								
			val = ind2.degree(this);
		}else{
			ind2 = old;
			val=this.getAlwaysMap().get(ind2);
		}	
		t2 = ind2.getTime();
		if (t1 < t2){			//from the beginning
			ind2 = ul.getFirst();						
			val = ind2.degree(this);
			t2 = ind2.getTime();
		}		
		while (t2<t1){
			ind2 = ul.getNext(ind2);
			if (ind2!=null){
				val = Math.min(val, ind2.degree(this));
				t2 = ind2.getTime();
			}
		}
		if (t2==t1){
			if (old != null && !old.equals(ind2)){
				this.getAlwaysMap().remove(old);
				this.getAlwaysMap().put(ind2, val);
			}else if (old == null)
				this.getAlwaysMap().put(ind2, val);
			return val;
		}		
		return 0.0;
	}

	/**
	 * Returns the last Individual, belonging to the given ULineEvent, 
	 * for which the sometime value has been evaluated.
	 * @param ul the given ULineEvent
	 * @return the last Individual
	 */
	private Individual getLastSometime(ULineEvent ul){
		Set<Individual> keys = this.getSometimeMap().keySet();
		Iterator<Individual> it = keys.iterator();
		while (it.hasNext()){
			Individual ind = (Individual)it.next();
			if (((DynamicInd)ind).getULineEvent().equals(ul))
				return ind;
		}		
		return null;
	}

	/**
	 * Returns the last Individual, belonging to the given ULineEvent,
	 * for which the always value has been evaluated.
	 * @param ul the given ULineEvent
	 * @return the last Individual
	 */
	private Individual getLastAlways(ULineEvent ul){
		Set<Individual> keys = this.getAlwaysMap().keySet();
		Iterator<Individual> it = keys.iterator();
		while (it.hasNext()){
			Individual ind = (Individual)it.next();
			if (((DynamicInd)ind).getULineEvent().equals(ul))
				return ind;
		}		
		return null;
	}

	/**
	 * Returns the evaluation of the logical expression [A until B(ind)],
	 * where A is this Concept, B is a given other Concept, ind is the Individual for which
	 * expression has to be evaluated; lastUntil is the given expression value for
	 * the last (previous) Individual with respect to ind.
	 * Operator Until has the weak and inclusive definition.
	 * Algorithm is incremental and dynamic: has last values memory. 
	 * @param b the Concept B
	 * @param ind the Individual
	 * @param lastUntil the last value
	 * @return the until evaluation
	 */
	public double until (Concept b, Individual ind, double lastUntil){
		System.out.println(this.getName()+"until"+b.getName()+"("+ind.getName()+"), with lastUntil: "+formatter.format(lastUntil));
		double some =this.sometime(ind);
		double bval = b.get(ind);
		System.out.println("sometime"+this.getName()+"("+ind.getName()+"): "+ formatter.format(some));
		System.out.println(b.getName()+"("+ ind.getName()+"): "+formatter.format(bval));
		double newUntil =Math.max(lastUntil, Math.min(bval,some ));
		System.out.println("new Until: " + formatter.format(newUntil));
		return newUntil;
	}

	/**
	 * Return the evaluation of the logical expression [A until B(ind)],
	 * where A is this Concept, B is a given Concept, ind is the Individual for which
	 * expression has to be evaluted.
	 * Operator Until has the weak and inclusive definition. 
	 * Algorithm is incremental and not dynamic: without memory, scan all the 
	 * ULineEvent history. 
	 * @param b the Concept B
	 * @param ind the Individual	
	 * @return the until evaluation
	 */
	public double until (Concept b, Individual ind){
		System.out.println(this.getName() +".until" +b.getName()+"("+ind.getName()+"):...");
		double until = 0.0;
		System.out.println("until: " + formatter.format(until));
		double time = ind.getTime();
		System.out.println("time: " + time);
		ULineEvent ul = ((DynamicInd)ind).getULineEvent();
		Individual xo = ul.getFirst();
		System.out.println("first_xo: " + xo.getName());
		double time0 = xo.getTime();
		System.out.println("time0: " + time0);
//		finch xo raggiunge ind
		while (time0 <= time || 						
//				oppure, se ind  il rappresentante di ul, xo raggiunge l'ultimo (calcolo su tutto ul)
//				( time == getSELF() && time0 != ul.getLast().getTime())){	
				( ind instanceof ULineInd && time0 != ul.getLast().getTime())){
//			if (time0 != getSELF()) {
			if (!(xo instanceof ULineInd)) {
				until = this.until(b, xo, until);	
				System.out.println("until: " + formatter.format(until));
			}			
			xo = ul.getNext(xo);
			if (xo==null)
				break;
			System.out.println("xo: " + xo.getName());
			time0 = xo.getTime();
			System.out.println("time0: " + time0);
		}			
		return until;
	}

	/**
	 * Add an Individual to this Concept, with a given membership value
	 * @param i the Individual
	 * @param d the membership value
	 * @return this Concept after adding Individual
	 */
	public Concept add(Individual i, Double d){
//		if (i.getTime()==getSELF()){
		if (i instanceof ULineInd){
			getULineMap().put(i, d);
		}
		else{
			getEvents().put(i, d);	
		}
		return this;
	}

//	/**	 
//	 * The cardinality of this Concept at an alpha level
//	 * @param alpha the alpha level
//	 * @return the Integer number cardinality of the alpha-set
//	 */
//	public Integer card(Double alpha) {
//		Set<Individual> keys =this.getEvents().keySet();
//		Iterator<Individual> iter= keys.iterator();
//		int card=0;
//		while(iter.hasNext()){
//			Individual key =(Individual) iter.next();
//			Double value=this.getEvents().get(key);
//			if (value>0.0 && value>=alpha){
//				card++;
//			}		
//		}	
//		return card;
//	}
	
	/**	 
	 * Gives this Concept cardinality at an alpha level. If mode = "any", evaluation 
	 * on all Events; if mode = "now", evaluation on now Events;
	 * if mode = "ul", evaluation on ULines
	 * @param alpha the alpha level
	 * @param mode the mode
	 * @return the integer number cardinality of the alpha-set
	 */
	public Integer card(Double alpha, String mode) {
		Set<Individual> keys =null;
		Iterator<Individual> iter;
		Double time = null;
		int card=0;
		if (mode.equals("any"))time=null;
		else if (mode.equals("now"))time=NOW;
		else if (mode.equals("ul"))time=SELF;
		else {
			System.out.println("error: set mode = \"any\" or \"now\" or \"ul\"" );
			return 0;
		}
		if(time==null ||time.equals(NOW))
			keys =this.getEvKeys();			//to be expanded for Statics
		else if(time.equals(SELF))
			keys =this.getULineMap().keySet();
		iter= keys.iterator();
		while(iter.hasNext()){
			Individual key =(Individual) iter.next();
			if (time == null || (((Double)key.getTime()).equals(time))){
				Double value=this.get(key);
				if (value>0.0 && value>=alpha)
					card++;
			}		
		}	
		return card;
	}
	/**
	 * Constructs an ArrayList of Points, containing Concept values, and,
	 * with it, produces a linear interpolation.
	 * @return a  new Point as the couple [angular coefficient, intercept]
	 */
	public Point interpole() {
		Set<Individual> keys =this.getEvents().keySet();
		Iterator<Individual> iter= keys.iterator();
		Double angCoeff, q, time, totaltime;
		Point p;
		ArrayList<Point> al = new ArrayList<Point>();
		totaltime = 20.0;					//to be set
		double size = keys.size();
		double unit = totaltime / size;
		time = 0.0;
		while(iter.hasNext()){
			Individual key =(Individual) iter.next();		
			Double value=this.getEvents().get(key);
			p = new Point(time, value);	
			al.add(p);
			time += unit;
		}	
		LinearInterpolation li = new LinearInterpolation(al);
		li.interpole();
		angCoeff = li.getM();
		q = li.getQ();
		return new Point(angCoeff, q);		
	}
	
	/**
	 * @return a TreeSet<Individual> as the this Concept EventInd set.  
	 */
	public Set<Individual> getEvKeys(){
		return (Set<Individual>) getEvents().keySet();
	}
	
	/**
	 * @return a TreeSet<Individual> as the this Concept ULineInd set.
	 */
	public Set<Individual> getUlKeys(){
		return (Set<Individual>) getULineMap().keySet();
	}

	/* (non-Javadoc)
	 * @see reasoner.FuzzySet#getLambda()
	 */
	/**
	 * Gives Lambda, the alpha values set, for the EventInds only
	 * @return Lambda as a TreeSet<Double>
	 */
	public TreeSet<Double> getLambda() {
		TreeSet<Double> l = new TreeSet<Double> (this.getEvents().values());		
		l.add(1.0);
		return l;
	}
	
	/**
	 * Gets the union set of alpha-values belonging to this Concept and to a 
	 * given other Concept, for the EventInds only
	 * @param c the Concept
	 * @return the union Lambda as a TreeSet<Double>
	 */
	public TreeSet<Double> totalLambda(Concept c) {
		TreeSet<Double> l = getLambda();
		if (c!=null){
			l.addAll(c.getLambda());
		}		
		return l;
	}
	
	/**
	 * Gives Lambda, the alpha values set, for the ULineInds only
	 * @return Lambda as a TreeSet<Double>
	 */
	public TreeSet<Double> getUlLambda() {
		TreeSet<Double> l = new TreeSet<Double> (this.uLineMap.values());		
		l.add(1.0);
		return l;
	}
	
	/**
	 * gets the total set of alpha-values belonging to this Concept and to
	 * a given other concept, for the ULine Individuals only
	 * @param c the given Concept
	 * @return Lambda as a TreeSet<Double>
	 */
	public TreeSet<Double> totalUlLambda(Concept c) {
		TreeSet<Double> l = getUlLambda();
		if (c!=null){
			l.addAll(c.getUlLambda());
		}		
		return l;
	}
	
//	/**
//	* @param f
//	* @return the sum with a FuzzySet
//	*/
//	public Concept sum(FuzzySet f){
//	//
//	return null;
//	}
//	/**
//	* @param f
//	* @return the difference with a FuzzySet
//	*/
//	public Concept diff(FuzzySet f){
//	//
//	return null;
//	}
//	/**
//	* @param f
//	* @return the product with a FuzzySet
//	*/
//	public Concept prod(FuzzySet f){
//	//
//	return null;
//	}
//	/**
//	* @param f
//	* @return the ratio with a FuzzySet
//	*/
//	public Concept ratio(FuzzySet f){
//	//
//	return null;
//	}

	/**
	 * The intersection between this Concept and a given second one
	 * @param c the given second Concept
	 * @return a new Concept intersection 
	 */
	public Concept intersection(Concept c) {
		Concept inter = new Concept(this.getName()+"_inter_"+c.getName(),getKb());
		inter.print();
		TreeSet<Individual> keys= new TreeSet<Individual>(getEvKeys());
		Individual key;
		Double aVal, cVal, minVal;		
		Iterator<Individual> it=keys.iterator();
		while (it.hasNext()){
			key=it.next();		
			System.out.println(((Individual)key).getName());
			if (c.containsKey(key)&& this.containsKey(key)){		
				aVal=(Double)this.get(key);	
				cVal=(Double)c.get(key);
				minVal=new Double(Math.min(aVal,cVal));	
//				if (minVal!=0){					//anche se val = 0->poniamo la cardinalit=0
				inter.add((Individual)key, minVal);	
				inter.getEvents().put((Individual)key, minVal);	
//				}
			}			
		}
		inter.print();
		inter.printUlines();
		return inter;
	}

//	/**
//	* @param f
//	* @return the union with a FuzzySets
//	*/
//	public Concept union(FuzzySet f){
//	//
//	return null;
//	}

	/**
	 * Tests on existence of a given Individual, as EventInd or ULineInd or StaticInd, in this Concept
	 * @param key the Individual
	 * @return true if key is in, otherwise false
	 */
	public boolean containsKey(Individual key) {
		if (key instanceof EventInd)
			return getEvents().containsKey(key);
		if (key instanceof ULineInd)
			return getULineMap().containsKey(key);
		if (key instanceof StaticInd)
			return getStaticMap().containsKey(key);
		return false;
	}
	
	/**
	 * Tests on existence of a given EventInd name in this Concept
	 * @param name the name as String
	 * @return true if any EventInd with this name is in, false otherwise
	 */
	public boolean containsIndivName(String name) {
		Iterator it = getEvKeys().iterator();
		while (it.hasNext()){
			if (((Individual)it.next()).getName().equals(name))
				return true;
		}
		return false;
	}

	/**
	 * Gives the membership value of the given Individual, 0.0 if it does not exist
	 * @param key the Individual
	 * @return the membership value
	 */
	public double get(Individual key) {
		if (getEvents().containsKey(key))
			return getEvents().get(key);
		else if (getULineMap().containsKey(key))
			return getULineMap().get(key);
		else if (getStaticMap().containsKey(key))
			return getStaticMap().get(key);
		else return 0.0;
	}
	
	/**
	 * Gives the subsumption value for this Concept in a given second one
	 * @param c the subsuming Concept 
	 * @return the value as double
	 */
	public double subsumed(Concept c, boolean fuzzy){
		if (fuzzy)
			return subsumed(new Quantifier("AlmostAll", getKb()),c); //fuzzy subsumption
		return subsumed(new Quantifier("All", getKb()),c);	//crisp subsumption


	}

	/**
	 * Returns the quantified subsumption value for this Concept in a given second one,
	 * given the quantifier.
	 * @param q the Quantifier
	 * @param c the subsuming Concept 
	 * @return the value as double
	 */
	public double subsumed(Quantifier q, Concept c){
		double sigma=0.0;
		if (q.getMaxval() <= 1.0){
			TreeSet<Double> lambda; 
			Concept ac = intersection(c);
			System.out.println("ac: ");
			ac.print();
			ac.printUlines();
			lambda = totalLambda(ac);
			System.out.println("lambda: ");
			q.print(lambda);
			Vector<Double> v = new Vector<Double>(lambda);	
			ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
			Double alfa1, alfa2, ratio, qvalue, diff;	
			Integer cardAcAlfa, cardAalfa;
			while(iter.hasPrevious()){
				alfa1 =(Double) iter.previous();
				cardAcAlfa= ac.card(alfa1 ,"any");
				System.out.println("card di "+ac.getName()+" (con alfa= "+ alfa1+"): "+cardAcAlfa);				 
				cardAalfa= card(alfa1 , "any");
				System.out.println("card di "+getName()+" (con alfa= "+ alfa1+"): "+cardAalfa);
				if (cardAcAlfa==0 && cardAalfa==0)											//
					ratio = 1.0;															//poniamo 0/0=1
				else
					ratio = cardAcAlfa.doubleValue()/cardAalfa.doubleValue();
				System.out.printf("rapporto (con alfa= %.2f): %.2f \n",alfa1,ratio);
				qvalue=q.qvalue(ratio);
				System.out.printf("qvalue : %.2f \n", qvalue);
				if (iter.hasPrevious()){
					alfa2=(Double) iter.previous();
					iter.next();//per tornare all'alfa corrente
				}else{
					alfa2=0.0;
				}
				System.out.printf("alfa1 : %.2f \n", alfa1);
				System.out.printf("alfa2 : %.2f \n", alfa2);
				diff=alfa1-alfa2;
				System.out.printf("diff : %.2f \n", diff);
				sigma += diff*qvalue;
				System.out.printf("sigma : %.2f \n", sigma);				
			}
		}
		return sigma;
	}

	/** 
	 * @return a String representation for this Concept, with all EventInds and values
	 */
	public String toString(){
		String s = "";
		Individual key;
		Double value;	
		Set<Individual> keys = getEvKeys();
		Iterator<Individual> iter= keys.iterator();
		s += "Concept: " + getName() + ": ";
		while(iter.hasNext()){
			key =(Individual)iter.next();
			value= (Double)get(key);	
			s += key.getName() + " --> " + formatter.format(value) + "; ";	
		}
		s += "\n";		
		return s;
	}	
	
	/** 
	 * Prints on the standard output a String representation for this Concept, 
	 * with all EventInds and values.
	 */
	public  void print() {
		System.out.println(this.toString());
	}
	/** 
	 * Prints on the standard output a String representation for this Concept, 
	 * with all ULineInds and values.
	 */
	public  void printUlines() {
		Set keys = getULineMap().keySet();
		Iterator iter= keys.iterator();
		System.out.println("Concept: " + getName());
		while(iter.hasNext()){
			Individual key =(Individual)iter.next();
			Double value= (Double)get(key);			
			System.out.printf("%s --> %.2f;  ",key.getName(), value);
		}
		System.out.println("");
	}	

	/**
	 * Returns a new Concept being  NextConcept for this Concept, for Individuals
	 * preceeding the given Individual. If the given Individual is ULineInd, then NextConcept 
	 * is for all the Individuals.
	 * @param ind the Individual
	 * @return the NextConcept
	 */
	public Concept getNextConcept(Individual ind){
		ULineEvent ul = ((DynamicInd)ind).getULineEvent();
		Concept next;
		double time,to;
		if (ind instanceof ULineInd)
			time = Double.MAX_VALUE;
		else
			time = ind.getTime();
		Individual xo, x1;
		xo = ul.getFirst();
		to = xo.getTime();
		x1 = ul.getNext(xo);		
		next = new Concept ("next"+this.getName(), getKb());
		while (null!=x1 && to <= time){
			next.add(xo, this.get(x1));
			xo = x1;
			to = xo.getTime();
			x1 = ul.getNext(xo);			
		}
//		next.add(xo, this.get(xo));		//prima possibilit: l'ultimo elemento  introdotto col valore inalterato
//		this.getMap().remove(xo);	//altra possibilit: eliminare l'ultimo elemento
		return next;
	}

	/** 
	 * @return the events
	 */
	public Map<Individual, Double> getEvents() {
		return events;
	}

	/**
	 * @param ev the events Map to set
	 */
	public void setEvents(Map<Individual, Double> ev) {
		this.events = ev;
	}	

	/**
	 * @return the uLineMap
	 */
	public Map<Individual, Double> getULineMap() {
		return uLineMap;
	}

	/**
	 * @param lineMap the uLineMap to set
	 */
	public void setULineMap(Map<Individual, Double> lineMap) {
		uLineMap = lineMap;
	}

//	/**
//	 * @return the a1
//	 */
//	private Concept getA1() {
//		return a1;
//	}
//
//	/**
//	 * @param a1 the a1 to set
//	 */
//	private void setA1(Concept a1) {
//		this.a1 = a1;
//	}
//
//	/**
//	 * @return the r
//	 */
//	private Concept getR() {
//		return r;
//	}
//
//	/**
//	 * @param r the r to set
//	 */
//	private void setR(Concept r) {
//		this.r = r;
//	}

	/**
	 * @return the kb
	 */
	public static KnowledgeBase getKb() {
		return kb;
	}

	/**
	 * @param kkb the kb to set
	 */
	public static void setKb(KnowledgeBase kkb) {
		kb = kkb;
		setNOW(kb.getNOW());
		setSELF(kb.getSELF());
	}

	/**
	 * @return the NOW
	 */
	public static double getNOW() {
		return NOW;
	}

	/**
	 * @param now the NOW to set
	 */
	public static void setNOW(double now) {
		NOW = now;
	}

	/**
	 * @return the SELF
	 */
	public static double getSELF() {
		return SELF;
	}

	/**
	 * @param self the SELF to set
	 */
	public static void setSELF(double self) {
		SELF = self;
	}

	/**
	 * @return the sometimeMap
	 */
	public Map<Individual, Double> getSometimeMap() {
		return sometimeMap;
	}

	/**
	 * @param sometimeMap the sometimeMap to set
	 */
	public void setSometimeMap(Map<Individual, Double> sometimeMap) {
		this.sometimeMap = sometimeMap;
	}

	/**
	 * @return the alwaysMap
	 */
	public Map<Individual, Double> getAlwaysMap() {
		return alwaysMap;
	}

	/**
	 * @param alwaysMap the alwaysMap to set
	 */
	public void setAlwaysMap(Map<Individual, Double> alwaysMap) {
		this.alwaysMap = alwaysMap;
	}

	/**
	 * @return the staticMap
	 */
	public Map<Individual, Double> getStaticMap() {
		return staticMap;
	}

	/**
	 * @param staticMap the staticMap to set
	 */
	public void setStaticMap(Map<Individual, Double> staticMap) {
		this.staticMap = staticMap;
	}
}
